/*
 * CDF.java
 */* NewGarden 
/*
/* Copyright  2005-2011 Stephan Pelikan and Steven Rogstad
/* contact:  pelikan@math.uc.edu
/*
/* This program is free software; you can redistribute it and/or modify
/* it under the terms of the GNU Library General Public License as published 
/* by  the Free Software Foundation; either version 3 of the License or
/* (at your option) any later version.
/*
/* This program is distributed in the hope that it will be useful, but
/* WITHOUT ANY WARRANTY; without even the implied warranty of
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/* GNU Library General Public License for more details.
/*
/* You should have received a copy of the GNU  General Public License
/* along with this program; see the file gpl-3.0.txt  If not, write to 
/* the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
/* Boston, MA  02111-1307 USA
 * 
 *
 */

/**
 *
 *CDF is Cumulative Distribution Function for a random variable
 *We're only really interested in drawing random values from it.
 * 
 */
import java.util.*;
import java.text.NumberFormat;

/**
 * CDF is a cumulative distribution function, or, rather,
 * a means for picking random values from a distribution
 * described by one.
 * 
 * It is basically just a SampledFunction with "x" and "y" interchanged
 * so that we can take the inverse of a random uniform [0,1] value
 * to get the desired variate
 *
 * Created on April 6, 2006, 6:49 PM
 * @author sep
 */
public class CDF extends SampledFunction {
    /**Turns reporting of errors on/off*/
    private static final boolean OurDEBUG = false;

    /** Creates a new instance of CDF */
    public CDF() {
        super();
    }

    /**
     * Overrides the SampledFunction cretor of the same name
     * @param xx array with x values of points on graph of the cdf
     * @param yy array with the y values of points on the graph of the cdf
     */
    public CDF(double[] xx, double[] yy) {
        super(yy, xx);
        assert yy[y.length - 1] == 1.0;
    }

    /**
     * addPoint() over rides the addPoint of SampledFunction to simply 
     * interchange x and y. We use addPoint() when constructing an CDF
     * by reading "functionpoint"
     * elements from an .xml file
     * @param x the x value
     * @param y the y value
     */
    @Override
    public void addPoint(double x, double y) {
        super.addPoint(y, x);
    }

    /**
     * For testing purposes
     * @param args not used
     */
    public static void main(String[] args) {
        double[] x = {0.0, 1.0, 2.0, 3.0, 4.0};
        double[] y = {0.0, 0.05, 0.1, 0.15, 1.0};
        double[] t = {0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 0.99,1.0};
        CDF cdf = new CDF(x, y);
        t = cdf.values(t);
        System.out.println(Arrays.toString(t));
        for (int i = 0; i < t.length; i++) {
            t[i] = Math.floor(t[i]);
        }
        System.out.println(Arrays.toString(t));
    }

    /**
     * toXML overrides the function from SampledFunction to
     * rechange the order of x and y
     * @return a String giving an XML version of this instance
     */
    @Override
    public String toXML() {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(2);
        StringBuffer sb = new StringBuffer("<CDF>" + "\n");
        for (int i = 0; i < x.length; i++) {
            sb.append("<functionpoint x=\"" + nf.format(y[i]) + "\" y=\"" + nf.format(x[i]) + "\"/>\n");
        }
        sb.append("</CDF>");
        return sb.toString();
    }

    @Override
    public String toXML(String name) {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(2);
        StringBuffer sb = new StringBuffer("<" + name + ">" + "\n");
        for (int i = 0; i < x.length; i++) {
            sb.append("<functionpoint x=\"" + nf.format(y[i]) + "\" y=\"" + nf.format(x[i]) + "\"/>\n");
        }
        sb.append("</" + name + ">\n");
        return sb.toString();
    }
}
