/*
 * Model.java
 *
 * Created on July 16, 2005, 12:39 AM
 * Last modified 17 May 2011 
 *
/* Copyright  2005-2011 Stephan Pelikan and Steven Rogstad
/* contact:  pelikan@math.uc.edu
/*
/* This program is free software; you can redistribute it and/or modify
/* it under the terms of the GNU Library General Public License as published 
/* by  the Free Software Foundation; either version 3 of the License or
/* (at your option) any later version.
/*
/* This program is distributed in the hope that it will be useful, but
/* WITHOUT ANY WARRANTY; without even the implied warranty of
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/* GNU Library General Public License for more details.
/*
/* You should have received a copy of the GNU  General Public License
/* along with this program; see the file gpl-3.0.txt  If not, write to 
/* the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
/* Boston, MA  02111-1307 USA
 */
/*
 *Changes
 *
 *April 2006  Introduced the Gnu GetOpt class and made command line options
 *to put turn lifetable on/off and to turn dump or whole population on/off
 *with options to put this output into text files.
 *
 *
 *??? Changed distribution mechanism from FruitDistance to DispersalDistance
 *  line which we can specify the CDF of dispersal distances Changed the DTD
 *   and the sample SimData.xml
 *
 *April 2009 Changed "Pollen_Distance" to a mechanism when we specifiy
 *the probabilities that pollen donor lies in different distance classes. Changed DTD
 *and sample SimData.xml
 *
 *
 *9 May 2006. Added ProgressBar and command line switches to turn it on
 *
 * 9 Jan 2008. I made sure that OffspringDistribution is read from the xml file,
 * changed to the new mechanism for regions and subregions, provided for
 * summary stats used on subregions, and made sure the FoundersF is read from the xml file.
 *
 * 15 Dec 2010 Wrote a simple command line options parser so we don't need to use
 * the getopt() library since I can't determine exactly whether it is LGPG or GPL'd
 * or what.
 **/

/**
 *
 * @author sep
 */
import Leslie.*;
//import org.xml.sax.*;
//import org.xml.sax.helpers.*;
import javax.xml.parsers.*;
import java.util.*;
import java.text.*;
import getopt.*;
import java.io.*;
import ProgressFrame.*;

public class Model {

    Random rnum;
    RandomValue rv;
    Leslie les;
    SimData sd;
    ArrayList population, expopulation;
    NumberFormat nf;
    boolean dump_pop = false;
    boolean do_life_table = false;
    boolean save_all_data = true;//record even out of region offspring WAS true
    boolean do_subregions = false;
    // boolean show_progress=false;
    static final boolean DEBUG = false;
    int[][][][] c_alleles;
    int[][] c_population;
    int[][] c_heterozygous;
    int[][] c_number_alleles;
    int[][][][] p_alleles;
    int[][] p_population;
    int[][] p_heterozygous;
    int[][] p_number_alleles;
    float[][] calculated_heterozygosity;
    //new 30 sept 06
    float[][] p_calculated_heterozygosity;
    /*
     * These are all new Dec 07 to allow summaries by region
     * They work the same as the above, but have an extra subscript
     * describing the region
     * 
     * 
     */
    //--------------------------------------------------
    int[][][][][] s_c_alleles;
    int[][][] s_c_population;
    int[][][] s_c_heterozygous;
    int[][][] s_c_number_alleles;
    int[][][][][] s_p_alleles;
    int[][][] s_p_population;
    int[][][] s_p_heterozygous;
    int[][][] s_p_number_alleles;
    float[][][] s_calculated_heterozygosity;
    float[][][] s_p_calculated_heterozygosity;
    //--------------------------------------------------
    private CommandLineParameters params;
    private LongOpt[] longopts = new LongOpt[6];
 

    private void initializeLongOpts() {
        longopts[0] = new LongOpt("help", LongOpt.NO_ARGUMENT, null, 'h');
        longopts[1] = new LongOpt("xmlfile", LongOpt.REQUIRED_ARGUMENT, null, 'x');
        longopts[2] = new LongOpt("dumpfile", LongOpt.OPTIONAL_ARGUMENT, null, 'd');
        longopts[3] = new LongOpt("progressbar", LongOpt.OPTIONAL_ARGUMENT, null, 'p');
        longopts[4] = new LongOpt("resultsfile", LongOpt.OPTIONAL_ARGUMENT, null, 'r');
        longopts[5] = new LongOpt("showNe", LongOpt.NO_ARGUMENT, null, 'n');
    }
private void newdoOptions(String [] args)
{
   for(int i=0;i<args.length;i++)
        {
            String ar=args[i];
            if(ar.startsWith("--"))
            {
                //System.out.println("Long option "+ ar.substring(2));
                new_do_long_option(ar.substring(2),args[i+1]);
                i++;
                continue;

            }
            if(ar.startsWith("-"))
            {
                //System.out.println("This option is "+ar.substring(0, 2));
                //System.out.println("Flag is "+ar.substring(1, 2));
                //System.out.println("Value is "+ ar.substring(2));
                new_do_option(ar.substring(1, 2),ar.substring(2));
            }
        }

}
 private void new_do_long_option(String name,String value)
    {
        //System.out.println("Long option flag = "+name+" value = "+value);
        if(name.equals("xmlfile")) 
        {
            System.out.println("You set XMLFile = "+value);
             if (value != null) {
                        //   System.out.println("You asked for XML parameters from file "+arg);
                        params.XMLfilename = value;
                    }
            else if (value == null || value.length()<=0) {
                        System.out.println("You need to specify a filename for this option");
                    }
        }
        if(name.equals("dumpfile"))
        {
            //if(value.isEmpty()) System.out.println("You set dumpfile = stdout");
            //else System.out.println("You set dumpfile = "+value);
            
              if (value != null) {
                        //System.out.println("You asked for a dump to file "+arg);
                        params.DUMPfilename = value;
                        params.ShowDump = true;
                    }
                    if (value == null || value.length()<=0) {
                        // System.out.println("You asked for a dump to stdout ");
                        params.DUMPfilename = null;
                        params.ShowDump = true;
                    }
        }
         if(name.equals("lifetable"))
         {
             System.out.println("You set lifetable option");
              if (value != null) {
                        // System.out.println("You asked for lifetable info to go to file "+arg);
                        params.ShowLifetable = true;
                        params.LIFETABLEfilename = value;
                    }
                    if (value == null || value.length()<=0) {
                        //System.out.println("You asked for lifetable info in stdout ");
                        params.ShowLifetable = true;
                        params.LIFETABLEfilename = null;
                    }
         }
         if(name.equals("progressbar"))
         {
             //System.out.println("You set progressbar option");
               params.ShowProgress = true;
         }
        if(name.equals("resultsfile"))
        {
            //System.out.println("You set resultsfile option to "+value);
              if (value!= null) {
                        //   System.out.println("You asked results written to "+arg);
                        params.RESULTSfilename = value;
                    }
                    if (value == null || value.length()<=0) {
                        System.out.println("If you use the '-r' commandline flag, you need to specify a filename for results file.\n");
                    }
        }
        if(name.equals("showNe")){
            //System.out.println("You set showNe option");
              params.ShowNe =true;
        }
    }
 private void new_do_option(String flag,String value)
    {
        if(flag.equals("h"))
        {
            System.out.println("You asked for help");
            System.out.println("The command line options are\n" +
                            "-h or --help: show this help message\n" +
                            "-x or --xmlfile <filename>: specify the XML parameter file\n" +
                            "-d or --dumpfile <filename>: put all the population data into the filename\n" +
                            "\t\t or to stdout if <filename> is not given\n" +
                            "-l or --lifetable <filename>: write Leslie lifetable to the file filename\n" +
                            "\t\t or to stdout is <filename> is not given.\n" +
                            "-p or --progressbar : show a progress bar on screen\n" +
                            "-v to label all bits of output with date-time and program version\n" +
                            "-r or --resultsfile <filename> to send the main results to the file filename\n" +
                            "-s to include a copy of the XML specified parameters in the results file\n" +
                            "-n or --showNe to show estimtates of Ne and their sd");
             System.exit(0);
        }
        if(flag.equals("x"))
        {
            //System.out.println("You set the XMFfilename to "+ value);
             if (value != null) {
                        //   System.out.println("You asked for XML parameters from file "+arg);
                        params.XMLfilename = value;
                    }
            else if (value == null) {
                        System.out.println("You need to specify a filename for this option");
                    }
        }
        if(flag.equals("d"))
        {
            //if(value.isEmpty()) //System.out.println("You set dumpfile to stdout");
            //else System.out.println("You set the dumpfile to "+ value);
              if (value != null) {
                        //System.out.println("You asked for a dump to file "+arg);
                        params.DUMPfilename = value;
                        params.ShowDump = true;
                    }
                    if (value == null || value.length()<=0) {
                        // System.out.println("You asked for a dump to stdout ");
                        params.DUMPfilename = null;
                        params.ShowDump = true;
                    }
        }
        if(flag.equals("l"))
        {
             //System.out.println("You asked for lifetable out put");
             if (value != null) {
                        // System.out.println("You asked for lifetable info to go to file "+arg);
                        params.ShowLifetable = true;
                        params.LIFETABLEfilename = value;
                    }
                    if (value == null || value.length()<=0) {
                        //System.out.println("You asked for lifetable info in stdout ");
                        params.ShowLifetable = true;
                        params.LIFETABLEfilename = null;
                    }
        }
        if(flag.equals("p"))
        {
             //System.out.println("You asked for progressbar");
             params.ShowProgress = true;
        }
        if(flag.equals("v"))
        {
             //System.out.println("You asked for verbose output");
             params.ShowVersion = true;
        }
        if(flag.equals("r"))
        {
             //System.out.println("You asked for results file "+ value);
              if (value!= null) {
                        //   System.out.println("You asked results written to "+arg);
                        params.RESULTSfilename = value;
                    }
                    if (value == null || value.length()<=0) {
                        System.out.println("If you use the '-r' commandline flag, you need to specify a filename for results file.\n");
                    }
        }
        if(flag.equals("s"))
        {
             //System.out.println("You asked for parameters in results file");
               params.ShowXML = true;
        }
         if(flag.equals("n"))
        {
             //System.out.println("You asked for Ne estimates in results file");
                params.ShowNe =true;
        }
    }
    private void doOptions(Getopt g, String[] args) {

        int c;
        String arg;
        while ((c = g.getopt()) != -1) {
            switch (c) {
                //options without  arguments
                case 'h':
                    System.out.println("You asked for help");
                    System.out.println("The command line options are\n" +
                            "-h or --help: show this help message\n" +
                            "-x or --xmlfile <filename>: specify the XML parameter file\n" +
                            "-d or --dumpfile <filename>: put all the population data into the filename\n" +
                            "\t\t or to stdout if <filename> is not given\n" +
                            "-l or --lifetable <filename>: write Leslie lifetable to the file filename\n" +
                            "\t\t or to stdout is <filename> is not given.\n" +
                            "-p or --progressbar : show a progress bar on screen\n" +
                            "-v to label all bits of output with date-time and program version\n" +
                            "-r or --resultsfile <filename> to send the main results to the file filename\n" +
                            "-s to include a copy of the XML specified parameters in the results file\n" +
                            "-n or --showNe to show estimtates of Ne and their sd");
                    System.exit(0);
                    break;
                case 'd':
                    arg = g.getOptarg();
                    if (arg != null) {
                        //System.out.println("You asked for a dump to file "+arg);
                        params.DUMPfilename = arg;
                        params.ShowDump = true;
                    }
                    if (arg == null) {
                        // System.out.println("You asked for a dump to stdout ");
                        params.DUMPfilename = null;
                        params.ShowDump = true;
                    }
                    break;
                case 'r':
                    arg = g.getOptarg();
                    if (arg != null) {
                        //   System.out.println("You asked results written to "+arg);
                        params.RESULTSfilename = arg;
                    }
                    if (arg == null) {
                        System.out.println("If you use the '-r' commandline flag, you need to specify a filename for results file.\n");
                    }
                    break;
                case 's':
                    params.ShowXML = true;

                    break;
                case 'n':
                    params.ShowNe =true;
                    break;
                case 'p':
                    //show_progress=true;
                    params.ShowProgress = true;
                    break;
                case 'x':
                    arg = g.getOptarg();
                    if (arg != null) {
                        //   System.out.println("You asked for XML parameters from file "+arg);
                        params.XMLfilename = arg;
                    }
                    if (arg == null) {
                        System.out.println("You need to specify a filename for this option");
                    }
                    break;
                case 'l':
                    arg = g.getOptarg();
                    if (arg != null) {
                        // System.out.println("You asked for lifetable info to go to file "+arg);
                        params.ShowLifetable = true;
                        params.LIFETABLEfilename = arg;
                    }
                    if (arg == null) {
                        //System.out.println("You asked for lifetable info in stdout ");
                        params.ShowLifetable = true;
                        params.LIFETABLEfilename = null;
                    }
                    break;

                //options with arguments
                case 'v':
                    // System.out.println("You asked for version information to be printed");
                    params.ShowVersion = true;
                    break;

                //
                case '?':
                    break; // getopt() already printed an error
                //
                default:
                //System.out.print("getopt() returned " + c + "\n");
            }
        }
        //IF XMLFILE isn't defined, it must be given on the command line after the options
        for (int i = g.getOptind(); i < args.length; i++) {
            System.out.println("Non option args element: " + args[i]);
        //Pick out the XML filename here.
        //  params.XMLfilename=args[(g.getOptind())];
        }
    }

    /**
     * Creates a new instance of Model
     */
    /**
     *This version parses the command line via getopts
     **/
    public Model(String[] args) {

        //HERE IS THE NEW COMMAND LINE PARSING
        params = new CommandLineParameters();
        initializeLongOpts();
        Getopt g = new Getopt("test", args, "snr:hpvl::x:d::", longopts);
        doOptions(g, args);

        //newdoOptions(args);

        rnum = new Random();
        rv = new RandomValue();
        rnum.setSeed(System.currentTimeMillis());
        getSimData(params.XMLfilename);
        sd.MakeFinal();
        if (sd.summaryregions != null && sd.summaryregions.Regions != null) {
            do_subregions = true;
        }
        dump_pop = params.ShowDump;
        do_life_table = params.ShowLifetable;
        nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(3);
        nf.setGroupingUsed(false);

        //allocate space to store results
        c_alleles = new int[sd.number_runs][sd.number_generations + 1][sd.number_loci][sd.max_alleles];
        c_population = new int[sd.number_runs][sd.number_generations + 1];
        c_heterozygous = new int[sd.number_runs][sd.number_generations + 1];
        c_number_alleles = new int[sd.number_runs][sd.number_generations + 1];
        calculated_heterozygosity = new float[sd.number_runs][sd.number_generations + 1];
        p_alleles = new int[sd.number_runs][sd.number_generations + 1][sd.number_loci][sd.max_alleles];
        p_population = new int[sd.number_runs][sd.number_generations + 1];
        p_heterozygous = new int[sd.number_runs][sd.number_generations + 1];
        p_number_alleles = new int[sd.number_runs][sd.number_generations + 1];
        p_calculated_heterozygosity = new float[sd.number_runs][sd.number_generations + 1];
        int number_summary_regions;      
        if (do_subregions) {
            number_summary_regions = sd.summaryregions.Regions.size();

            s_c_alleles = new int[number_summary_regions][sd.number_runs][sd.number_generations + 1][sd.number_loci][sd.max_alleles];
            s_c_population = new int[number_summary_regions][sd.number_runs][sd.number_generations + 1];
            s_c_heterozygous = new int[number_summary_regions][sd.number_runs][sd.number_generations + 1];
            s_c_number_alleles = new int[number_summary_regions][sd.number_runs][sd.number_generations + 1];
            s_calculated_heterozygosity = new float[number_summary_regions][sd.number_runs][sd.number_generations + 1];
            s_p_alleles = new int[number_summary_regions][sd.number_runs][sd.number_generations + 1][sd.number_loci][sd.max_alleles];
            s_p_population = new int[number_summary_regions][sd.number_runs][sd.number_generations + 1];
            s_p_heterozygous = new int[number_summary_regions][sd.number_runs][sd.number_generations + 1];
            s_p_number_alleles = new int[number_summary_regions][sd.number_runs][sd.number_generations + 1];
            s_p_calculated_heterozygosity = new float[number_summary_regions][sd.number_runs][sd.number_generations + 1];
       
        }

       
        runModel();
    }

   
    public void getSimData(String xmlfilename) {
        SimDataReader my_mb = null; //a model builder for parsing .xml
        try {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser saxParser = factory.newSAXParser();
            org.xml.sax.XMLReader parser = saxParser.getXMLReader();
            my_mb = new SimDataReader();
            parser.setContentHandler(my_mb);
            try {
                parser.parse(xmlfilename);
            } catch (Exception e) {
                System.out.println("Parsing error in SimData(xmlfilename) " + e.toString());
            }
        } catch (Exception e) {
            System.out.println("Problem getting a parser in SimDataReader()");
        }
        sd = my_mb.getSD();
    //sd.MakeFinal();
    }

    /**
     *do life table calculations with the leslie model
     *put the results to stdout or a file depending on what the commandline
     *asked for
     **/
    void doLifeTable() {
        PrintStream ourPS;
        if (params.LIFETABLEfilename == null) {
            ourPS = System.out;
        } else {
            try {
                ourPS = new PrintStream(new File(params.LIFETABLEfilename));
            } catch (Exception e) {
                System.out.println(e.toString());
                ourPS = System.out;
            }
        }
        if (params.ShowVersion && params.LIFETABLEfilename != null) {
            ourPS.println(params.toString(true));
        }
        ourPS.println("LIFE TABLE CALCULATIONS");
        if (sd == null) {
            System.out.println("Cant do lifetable calculations without a SimData");
        }
        //build an array of reproduction rates
        //first, how many age classes --- what is largest age
        //for which mortality and reproduction is defined.
        double x1 = sd.fruit_production.x[sd.fruit_production.x.length - 1];
        double x2 = sd.mortality.x[sd.mortality.x.length - 1];
        double maxage = x1 < x2 ? x2 : x1;
        int ourdim = (int) maxage;
        double[] rrate = new double[(int) maxage + 1];
        //get reproduction rates according to current method
        for (int i = 0; i <= maxage; i++) {
            rrate[i] = sd.fruit_production.value(i);
            int n_offspring;
            if (sd.off_dist_method.equals("bracket")) {
                double fp = sd.fruit_production.value(i);
                //we pick between a=floor(fp) and b=floor(fp)+1 so we
                //end up with the correct average
                //p a +(1-p)b =fp
                //so p(a-b)=fp-b
                //p =(b-fp)/(b-a)=b-fp
                int a = (int) Math.floor(fp);
                if ((double) a == fp) {
                    n_offspring = a;
                } else {
                    int b = a + 1;
                    if (rnum.nextDouble() < (double) b - fp) {
                        n_offspring = a;
                    } else {
                        n_offspring = b;
                    }
                }
            } else if (sd.off_dist_method.equals("poisson")) {
                //System.out.println("poisson reproduction isn't implemented yet!");
                //System.exit(0);
                double fp = sd.fruit_production.value(i);
                n_offspring = (int) rv.Poisson(fp, 1)[0];


            } else //i.e. "round" or unspecified
            {
                n_offspring = (int) Math.round(sd.fruit_production.value(i));
            }
            rrate[i] = n_offspring;
        }
        double[] mrate = new double[(int) maxage];
        for (int i = 0; i < maxage; i++) {
            mrate[i] = sd.mortality.value((double) i);
        }

        les = new Leslie(rrate, mrate);
        if (DEBUG) {
            ourPS.println("Here are the reproduction rates:");
            for (int i = 0; i < rrate.length; i++) {
                ourPS.print(" " + nf.format(rrate[i]));
            }
            ourPS.println();

            ourPS.println("Here are the mortality nfrates:");
            for (int i = 0; i < mrate.length; i++) {
                ourPS.print(" " + nf.format(mrate[i]));
            }
            ourPS.println();

        }
        ourPS.println(les.calculate());
        //now the approximate calculation
        //we begin by getting initial population age distribution.
        double[] n0 = new double[rrate.length];
        Iterator it = sd.plants.iterator();
        while (it.hasNext()) {
            Plant pl = (Plant) it.next();
            n0[pl.dob]++;
        }
        ourPS.println(les.approximate(n0, 5));
    }

    void makeInitialPopulation() {
        //loop over sd.plants creating a real plant, giving it alleles
        Plant.id_no = sd.plants.size();
        Iterator it = sd.plants.iterator();
        mainloop:
        while (it.hasNext()) {
            Plant pl = (Plant) it.next();
            Plant ourplant = new Plant(pl.allele.length, -1 * pl.dob, pl.location);
            ourplant.parent1 = ourplant.parent2 = pl.my_id_no;
            ourplant.female = pl.female;
            ourplant.founder = true;
            //NEW on 2 April 2006
            //If this location is already occupied, ignore this founder
            //and print a message for the user
            Iterator popit = population.iterator();
            while (popit.hasNext()) {
                Plant pp = (Plant) popit.next();
                if (pp.location.equals(ourplant.location)) {
                    System.out.println("You've got two founders at one location.\nI'll ignore the second one. location =" + ourplant.location.toString() + "\n");
                    continue mainloop;
                }
            }
            //pick the alleles for ourplant
            //loop to tell each locus the value of F
            Iterator li = sd.loci.iterator();
            while (li.hasNext()) {
                Locus l = (Locus) li.next();
                l.setF(sd.foundersf);
            }
            for (int loc = 0; loc < sd.loci.size(); loc++) {
                int[] genes = ((Locus) sd.loci.get(loc)).pickGenotype(rnum);
                for (int a = 0; a < 2; a++) {
                    ourplant.allele[loc][a] = genes[a];
                }
            }

            population.add(ourplant);
        }

    }
/** 
 * Create a string to print list the whole population
 * @return the string
 */
    public String showPopulation() {
        Iterator it;
        StringBuffer sb = null;
        if (population.size() > 0) {
            Plant temp = (Plant) population.get(0);
            sb = new StringBuffer(temp.CSVHeader());
            it = population.iterator();
            while (it.hasNext()) {
                Plant p = (Plant) it.next();
                sb.append(p.toCSVString(sd.run_number));
            }
        }
        if (expopulation.size() > 0) {
            if (sb == null) {
                Plant temp = (Plant) expopulation.get(0);
                sb = new StringBuffer(temp.CSVHeader());
            }
            it = expopulation.iterator();
            while (it.hasNext()) {
                Plant p = (Plant) it.next();
                sb.append(p.toCSVString(sd.run_number));
            }
        }
        return sb.toString();
    }
/**
 * 
 * @param a --- and array
 * @return --- an integer indexing a random member of the array
 */
    int random_member(ArrayList a) {
        return (int) Math.floor((a.size()) * rnum.nextDouble());
    }

    /**
     *nextYear() implements the next round of mortality, breeding, dispersal
     *
     */
    void nextYear() {
       
        Iterator it = population.iterator();
        //loop over everyone remaining to generate tentative list of offspring
        ArrayList templist = new ArrayList();
        it = population.iterator();
        loop:
        while (it.hasNext()) {
            Plant fparent = (Plant) it.next(); //pick a female parent
            Plant mparent = null;

            //if we are dioecious, only look at females
            if (sd.dioeceous && !fparent.female) {
                continue loop;
            }

            //How many offspring (fruit) should this parent produce?
            //The number of offspring is determined by the age of the
            //(female) parent
            int age = sd.current_year - fparent.dob;
           
            int n_offspring = 0;
            //THE NEXT LINES ALLOW NEW METHODS
            if (sd.off_dist_method.equals("bracket")) {
                double fp = sd.fruit_production.value(age);
                int a = (int) Math.floor(fp);
                if ((double) a == fp) {
                    n_offspring = a;
                } else {
                    int b = a + 1;
                    if (rnum.nextDouble() < (double) b - fp) {
                        n_offspring = a;
                    } else {
                        n_offspring = b;
                    }
                }
            } else if (sd.off_dist_method.equals("poisson")) {
                double fp = sd.fruit_production.value(age);
                n_offspring = (int) rv.Poisson(fp, 1)[0];

            } else //i.e. "round" or unspecified
            {
                n_offspring = (int) Math.round(sd.fruit_production.value(age));
            }
            //END OF THE NEW METHODS FOR NUMBER OF OFFSPRING

           
            //NEW POLLEN PROCEDURES 7 Apr 2006
          
            int number_distance_ranges = sd.pollen_distances.length + 1;
            DPD[] DPDs = new DPD[number_distance_ranges];
            for (int i = 0; i < DPDs.length; i++) {
                DPDs[i] = new DPD();
            }

            //loop over population to put each potential pollen source in the 
            //correct DPD
            Iterator popit = population.iterator();
            while (popit.hasNext()) {
                Plant pp = (Plant) popit.next();


                //CHECK IF WE WANT TO CONSIDER THIS PP BASED ON SEX, RANDOM MATING ETC
                //find to which distance range this pp belongs
                float ourDist = pp.location.dist(fparent.location);
                int distance_class = locateClass(ourDist, sd.pollen_distances);
                //continue loop if distance_class is too big
                //look up the age-dependent (relative) chance of being
                //a pollen donor for thie potential parent pp
                float prob = (float) sd.pollen_production.value(sd.current_year - pp.dob);
                if (prob > 0) // add to correct DPD
                {
                    if (pp.my_id_no == fparent.my_id_no && sd.random_mating) {
                        DPDs[distance_class].addPoint(prob, pp);
                    }
                    if (pp.my_id_no != fparent.my_id_no)//enforce outcrossing
                    {
                        if (sd.dioeceous && !pp.female) {
                            DPDs[distance_class].addPoint(prob, pp);
                        } else {
                            DPDs[distance_class].addPoint(prob, pp);
                        }
                    }
                }
            }
            //NORMALIZE THE DPDs that are nonempty
            for (int i = 0; i < DPDs.length; i++) {
                //new
                //didn't test before we didn't test before, just made DPD
                //do some specific normalization if it was empty
                if (DPDs[i].number_points() > 0) {
                    DPDs[i].normalize();
                }
            }


            //END NEWPOLLEN PROCEDURE
           
            //ACTUALLY GENERATE POTENTIAL OFFSPRING, RESELECTING A POLLEN SOURCE FOR EACH
            offloop:
            for (int offsp = 0; offsp < n_offspring; offsp++) {
                mparent = null;
                //outcross or selfing --- pick alleles for offspring
               
                //HERE WE SELECT THE POLLEN SOURCE ***NEW***
                if (sd.dioeceous || (rnum.nextDouble() > sd.selfing_rate))//force an outcross
                {
                    //NEW POLLEN PROCEDURE
                    //DETERMINE A DISTANCE CLASS AND THEN PICK A mparent
                    int dist_class = (int) Math.floor(sd.pollen_distance_probabilities.pickOne(rnum.nextDouble()));
                    if (DEBUG) {
                        assert DPDs[dist_class] != null : "null prob dist in nextYear()!";
                    }
                    if (DPDs[dist_class].number_points() <= 0) {
                        continue offloop;
                    }
                    mparent = (Plant) DPDs[dist_class].pickOne(rnum.nextDouble());
                //END NEW POLLEN PROCEDURE
                } else//self
                {
                    mparent = fparent;
                }
                //DONE SELECTING mparent ***END NEW***
                if (DEBUG) {
                    assert mparent != null : "mparent == null in nextYear()";
                }
                if (mparent == null) {
                    continue offloop; //shouldn't ever need to do this
                }
                Plant offspring = new Plant(sd.loci.size());
                offspring.parent1 = fparent.my_id_no;
                offspring.parent2 = mparent.my_id_no;
                offspring.dob = sd.current_year + 1;
                offspring.female = rnum.nextBoolean();

                                             

                //NEW LOCATION SELECTION
                //Pick the distance d from sd.DispersalDistribution
                //offspring.location=chooseLocation(fparent.location, d);
                //

                offspring.location = chooseLocation(fparent.location, (int) Math.floor(sd.dispersal_distribution.value(rnum.nextDouble())));

                ///  if(!sd.region.inRegion(tx, ty)) {//tx=offspring.location.X etc.
                if (!sd.region.inRegion((float) offspring.location.X, (float) offspring.location.Y)) {
                    //ELIMINATE THE SAVE_ALL_DATA loops WE DONT NEED THEM
                    if (save_all_data) {
                        //put the guy in expopulation and continue the loop
                        offspring.dod = sd.current_year;
                        expopulation.add(offspring);
                    }
                    continue offloop;
                }
                //pick alleles for offspring
                //loop over loci
                for (int loc = 0; loc < sd.loci.size(); loc++) {
                   
                    boolean choice = rnum.nextBoolean();
                    int ac;//fparent allele to choose
                    if (choice) {
                        ac = 0;
                    } else {
                        ac = 1;
                    }
                    offspring.allele[loc][0] = fparent.allele[loc][ac];

                    choice = rnum.nextBoolean();

                    if (choice) {
                        ac = 0;
                    } else {
                        ac = 1;
                    }
                    offspring.allele[loc][1] = mparent.allele[loc][ac];
                }

                //add the offspring to temmplist
                templist.add(offspring);
            }

        }

        //THIS iS THE NEW LOCATION FOR MORTALITY OF PARENTAL GENERATION
        //Everyone currently in the population
        //gets killed at age specific rates
        Iterator ourit = population.iterator();
        while (ourit.hasNext()) {
            Plant p = (Plant) ourit.next();
            int age = sd.current_year - p.dob;
            double mortality = sd.mortality.value(age);
            if (rnum.nextDouble() < mortality) {
                ourit.remove();
                p.dod = sd.current_year;
                expopulation.add(p);
            }
        }

       
        /** NEW METHOD
         *  a little faster in a test with high population density
         */
        //Loop over all new plants to remove any near a parent
        //for each offspring we iterate through population
        //perhaps ending early
        /**********************************************/
        Iterator ourTLI = templist.iterator();
        while (ourTLI.hasNext()) {
            Plant ourOff = (Plant) ourTLI.next();
            Iterator ourPLI = population.iterator();
            while (ourPLI.hasNext()) {
                Plant ourPar = (Plant) ourPLI.next();
                if (ourOff.location.equals(ourPar.location)) {
                    ourTLI.remove();
                    break; //iteration over parent list
                }
            }
        }
        //Randomize the order of the offspring
        ArrayList newtemplist = (ArrayList) templist.clone();
        int[] permutedIndices = rv.pickKfromM(templist.size(), templist.size());
        for (int ii = 0; ii < permutedIndices.length; ii++) {
            newtemplist.set(ii, templist.get(permutedIndices[ii]));
        }
        //Remove the first offspring to the population
        //(if we don't kill it) NOW WE DON'T KILL OFF age==0 HERE
        //and then remove any offspring close to it
        templist = newtemplist; //frees old templist memory
        while (templist.size() > 0) {
            Plant ourOff = (Plant) templist.get(0);
            //eliminated the mortality loop Feb 2009
            //double mortality = sd.mortality.value(0);
            //if (rnum.nextDouble() > mortality) {

                population.add(ourOff);
            //}
            ourTLI = templist.iterator();
            while (ourTLI.hasNext()) {
                Plant tplant = (Plant) ourTLI.next();
                if (ourOff.location.equals(tplant.location)) {
                    ourTLI.remove();
                }


            }
        }
    /******************************************/
    }

    private Location chooseLocation(Location parent_location, int d) {
        //There are (2d+1)^2 points at distance \le d
        //and (2d-1)^2 at distance \le d-1 and therefore
        //4d^2+4d+1 -(4d^2-4d+1) = 8d points at distance d
        //numbering them clockwise around the square from 0=ipper left the
        //corners are at 0, 2d, 4d, and 6d
        // x= uniformrandomint(0\le x \le 8d-1) RandomInt(0,8d-1)
        //if 0\le x \le 2d  point<- (px-d+x, py+d)
        //if 2d\le x \le 4d point <- (px+d, py+d - (x-2d))
        //if 4d\le x \le 6d  point<= (px+d-(x-4d)   ,py-d)
        //if 6d \le x \le 8d  point <- (px-d,py-d+(x-6d))
        int x;
        if (d == 0) {
            x = 0;
        } else {
            x = rv.RandomInt(0, 8 * d - 1);
        }

        // if(DEBUG)
        // {
        //     System.out.println("d = "+d+" x = "+x);
        // }

        if (0 <= x && x <= 2 * d) {
            return parent_location.add(new Location(x - d, d));
        } else if (2 * d <= x && x <= 4 * d) {
            return parent_location.add(new Location(d, 3 * d - x));
        } else if (4 * d <= x && x <= 6 * d) {
            return parent_location.add(new Location(5 * d - x, -d));
        } else if (6 * d <= x && x <= 8 * d) {
            return parent_location.add(new Location(-d, x - 7 * d));
        } else {
            if (DEBUG) {
                System.out.println("This shouldn't happen");
            }
            return null;
        }

    }

    public void runModel() {
        //System.out.println(""+sd.run_number+" "+sd.current_year);
        // if(do_life_table) doLifeTable();
        if (params.ShowLifetable) {
            doLifeTable();
        }
        String[] labels = {"NewGarden Progress", "simulation: run #", "run: generation#"};
        ProgressFrame pf = null;
        if (params.ShowProgress) {
            pf = new ProgressFrame(2, labels);
        }
        for (sd.run_number = 0; sd.run_number < sd.number_runs; sd.run_number++) {
            if (params.ShowProgress) {
                pf.setValue(0, pf.makePercent(sd.run_number, sd.number_runs - 1));
                pf.setValue(1, 0);
            }
            population = new ArrayList();
            expopulation = new ArrayList();
            makeInitialPopulation();
            //System.out.println(showPopulation());
            for (sd.current_year = 0; sd.current_year < sd.number_generations - 1; sd.current_year++) {
                //System.out.println(""+sd.run_number+" "+sd.current_year);
                nextYear();
                if (params.ShowProgress) {
                    pf.setValue(1, pf.makePercent(sd.current_year, sd.number_generations - 2));
                }
            }
            //if we want each simulations population printed, here is where to do it
            //control this with a command line switch
            // if(dump_pop)  System.out.println( showPopulation());
            if (params.ShowDump) {
                if (params.DUMPfilename == null) {
                    System.out.println(showPopulation());
                } else {
                    if (params.dumpfilewriter == null) {
                        File f = new File(params.DUMPfilename);
                        BufferedWriter br = null;
                        try {
                            br = new BufferedWriter(new FileWriter(f));
                        } catch (Exception e) {
                            System.out.println(e.toString());
                        }

                        params.dumpfilewriter = new PrintWriter(br, true);
                        if (params.ShowVersion && params.DUMPfilename != null) {
                            params.dumpfilewriter.println(params.toString(true));
                        }
                    }
                    params.dumpfilewriter.println(showPopulation());
                }
            }

            makeSummary();

            if (do_subregions) {
                makeRegionalSummary();
            }

        }
        //we should put this to stdout or a file, depending on what commandline asks us to do
        if (params.RESULTSfilename != null) {
            File resultsfile = new File(params.RESULTSfilename);
            try {
                FileWriter fw = new FileWriter(resultsfile);
                BufferedWriter br = new BufferedWriter(fw);
                PrintWriter pw = new PrintWriter(br, true);
                if (params.ShowVersion) {
                    pw.println(params.toString(false));
                }
                if (params.ShowXML) {
                    pw.println(sd.toString());
                }
                pw.println(showSummary(0));

                if (do_subregions) {
                    pw.println(showRegionalSummary());
                }
                fw.close();
            } catch (Exception e) {
                System.out.println(e.toString());
            }
        } else {
            if (params.ShowVersion) {
                System.out.println(params.toString(true));
            }
            if (params.ShowXML) {
                System.out.println(sd.toString());
            }
            System.out.println(showSummary(0));
            if (do_subregions) {
                System.out.println(showRegionalSummary());
            }

        //and then show summaries by region if there are any
        }
        if (params.ShowProgress) {
            pf.done();
        }
        System.exit(0);
    }

    /**
     *makeSummary is called after each run
     *It reads through the entire population (and expopulation)
     *to accumulate information on plants that were alive during each year of the simulation
     *and also by cohort --- counting only those plants produced in a given year
     *(and therefore of a given age =1,2,3...)
     *The only plants that  need to be treated specially are the founders whose
     *dob is fudged to give them the desired age at the start of the run.
     *This will appear as "age=0" in the summary, but the represent the founders not age=0
     **/
    void makeSummary() {
        //these will be global variables initialized when we start

        Iterator it = population.iterator();
        loop:
        while (it.hasNext()) {
            Plant p = (Plant) it.next();
            if (p.dod >= 0 && p.dod < p.dob) {
                continue loop;
            }
            int cohort;
            if (p.founder) {
                cohort = 0;
            } else {
                cohort = p.dob;
            }
            c_population[sd.run_number][cohort]++;//size of cohort
            for (int loc = 0; loc < sd.number_loci; loc++) {//count alleles of each type
                c_alleles[sd.run_number][cohort][loc][p.allele[loc][0]]++;
                c_alleles[sd.run_number][cohort][loc][p.allele[loc][1]]++;
                if (p.allele[loc][0] != p.allele[loc][1]) {//and if this plant is hetero at this locus
                    c_heterozygous[sd.run_number][cohort]++;
                }
            }
            int a, top;
            if (p.dod < 0) {
                top = sd.number_generations;
            } else {
                top = p.dod;
            }
            for (a = cohort; a <= top; a++) {
                p_population[sd.run_number][a]++;
                for (int loc = 0; loc < sd.number_loci; loc++) {
                    p_alleles[sd.run_number][a][loc][p.allele[loc][0]]++;
                    p_alleles[sd.run_number][a][loc][p.allele[loc][1]]++;
                    if (p.allele[loc][0] != p.allele[loc][1]) {
                        p_heterozygous[sd.run_number][a]++;
                    }
                }
            }
        }
        it = expopulation.iterator();
        looploop:
        while (it.hasNext()) {
            Plant p = (Plant) it.next();
            if (p.dod >= 0 && p.dod < p.dob) {
                continue looploop;
            }
            int cohort;
            if (p.founder) {
                cohort = 0;
            } else {
                cohort = p.dob;
            }
            c_population[sd.run_number][cohort]++;
            for (int loc = 0; loc < sd.number_loci; loc++) {
                c_alleles[sd.run_number][cohort][loc][p.allele[loc][0]]++;
                c_alleles[sd.run_number][cohort][loc][p.allele[loc][1]]++;
                if (p.allele[loc][0] != p.allele[loc][1]) {
                    c_heterozygous[sd.run_number][cohort]++;
                }
            }
            int a, top;
            if (p.dod < 0) {
                top = sd.number_generations;
            } else {
                top = p.dod;
            }
            for (a = cohort; a <= top; a++) {
                p_population[sd.run_number][a]++;
                for (int loc = 0; loc < sd.number_loci; loc++) {
                    p_alleles[sd.run_number][a][loc][p.allele[loc][0]]++;
                    p_alleles[sd.run_number][a][loc][p.allele[loc][1]]++;
                    if (p.allele[loc][0] != p.allele[loc][1]) {
                        p_heterozygous[sd.run_number][a]++;
                    }
                }

            }
        }
        //now we can find the average of the heterozygosities over all
        //the loci for each ageclass.
        /*
         *for each locus h[locus]=1-sum((allele[cohort][locus][al]/(2*chort)))^2)
         *avghet=sum(h[locus]/number_loci
         *calculated_heterozygosity[run_number][ageclass]=avghet
         *
         *That is, the calculated H is assuming HW
         */
        for (int cohrt = 0; cohrt < sd.number_generations + 1; cohrt++) {
            float chsize = c_population[sd.run_number][cohrt];
            if (chsize > 0) {
                float h = 0;
                for (int loc = 0; loc < sd.number_loci; loc++) {
                    float ss = 0;
                    for (int al = 0; al < ((Locus) sd.loci.get(loc)).number_alleles(); al++) {
                        ss += c_alleles[sd.run_number][cohrt][loc][al] * c_alleles[sd.run_number][cohrt][loc][al] / (4 * chsize * chsize);
                    }
                    h += 1 - ss;
                }
                calculated_heterozygosity[sd.run_number][cohrt] = h / sd.number_loci;
            }
        }

        //We need that same thing --- calculated H ---for population at each year
        //new 30 sept 2006
        for (int ourage = 0; ourage < sd.number_generations + 1; ourage++) {
            float psize = p_population[sd.run_number][ourage];
            if (psize > 0) {
                float h = 0;
                for (int loc = 0; loc < sd.number_loci; loc++) {
                    float ss = 0;
                    for (int al = 0; al < ((Locus) sd.loci.get(loc)).number_alleles(); al++) {
                        ss += p_alleles[sd.run_number][ourage][loc][al] * p_alleles[sd.run_number][ourage][loc][al] / (4 * psize * psize);
                    }
                    h += 1 - ss;
                }
                p_calculated_heterozygosity[sd.run_number][ourage] = h / sd.number_loci;
            }
        }
    }

  
    public void makeRegionalSummary() {
        //Loop over the summaryregions
        //for each, make new pop and expop containing
        //just the plants in the given regions
        //and do standard summary process with
        //region index set to RN
        ArrayList pop, expop;
        int number_summary_regions = sd.summaryregions.Regions.size();
        int RN;
        for (RN = 0; RN < number_summary_regions; RN++) {
            Region ourRegion = (Region) sd.summaryregions.Regions.get(RN);
            pop = new ArrayList();
            expop = new ArrayList();
            Iterator it = population.iterator();
            while (it.hasNext()) {
                Plant p = (Plant) it.next();
                if (ourRegion.inRegion(p.location.X, p.location.Y)) {
                    pop.add(p);
                }
            }
            it = expopulation.iterator();
            while (it.hasNext()) {
                Plant p = (Plant) it.next();
                if (ourRegion.inRegion(p.location.X, p.location.Y)) {
                    expop.add(p);
                }
            }

            it = pop.iterator();
            loop:
            while (it.hasNext()) {
                Plant p = (Plant) it.next();
                if (p.dod >= 0 && p.dod < p.dob) {
                    continue loop;
                }
                int cohort;
                if (p.founder) {
                    cohort = 0;
                } else {
                    cohort = p.dob;
                }
                s_c_population[RN][sd.run_number][cohort]++;//size of cohort
                for (int loc = 0; loc < sd.number_loci; loc++) {//count alleles of each type
                    s_c_alleles[RN][sd.run_number][cohort][loc][p.allele[loc][0]]++;
                    s_c_alleles[RN][sd.run_number][cohort][loc][p.allele[loc][1]]++;
                    if (p.allele[loc][0] != p.allele[loc][1]) {//and if this plant is hetero at this locus
                        s_c_heterozygous[RN][sd.run_number][cohort]++;
                    }
                }
                int a, top;
                if (p.dod < 0) {
                    top = sd.number_generations;
                } else {
                    top = p.dod;
                }
                for (a = cohort; a <= top; a++) {
                    s_p_population[RN][sd.run_number][a]++;
                    for (int loc = 0; loc < sd.number_loci; loc++) {
                        s_p_alleles[RN][sd.run_number][a][loc][p.allele[loc][0]]++;
                        s_p_alleles[RN][sd.run_number][a][loc][p.allele[loc][1]]++;
                        if (p.allele[loc][0] != p.allele[loc][1]) {
                            s_p_heterozygous[RN][sd.run_number][a]++;
                        }
                    }
                }
            }
            it = expop.iterator();
            looploop:
            while (it.hasNext()) {
                Plant p = (Plant) it.next();
                if (p.dod >= 0 && p.dod < p.dob) {
                    continue looploop;
                }
                int cohort;
                if (p.founder) {
                    cohort = 0;
                } else {
                    cohort = p.dob;
                }
                s_c_population[RN][sd.run_number][cohort]++;
                for (int loc = 0; loc < sd.number_loci; loc++) {
                    s_c_alleles[RN][sd.run_number][cohort][loc][p.allele[loc][0]]++;
                    s_c_alleles[RN][sd.run_number][cohort][loc][p.allele[loc][1]]++;
                    if (p.allele[loc][0] != p.allele[loc][1]) {
                        s_c_heterozygous[RN][sd.run_number][cohort]++;
                    }
                }
                int a, top;
                if (p.dod < 0) {
                    top = sd.number_generations;
                } else {
                    top = p.dod;
                }
                for (a = cohort; a <= top; a++) {
                    s_p_population[RN][sd.run_number][a]++;
                    for (int loc = 0; loc < sd.number_loci; loc++) {
                        s_p_alleles[RN][sd.run_number][a][loc][p.allele[loc][0]]++;
                        s_p_alleles[RN][sd.run_number][a][loc][p.allele[loc][1]]++;
                        if (p.allele[loc][0] != p.allele[loc][1]) {
                            s_p_heterozygous[RN][sd.run_number][a]++;
                        }
                    }

                }
            }

            for (int cohrt = 0; cohrt < sd.number_generations + 1; cohrt++) {
                float chsize = s_c_population[RN][sd.run_number][cohrt];
                if (chsize > 0) {
                    float h = 0;
                    for (int loc = 0; loc < sd.number_loci; loc++) {
                        float ss = 0;
                        for (int al = 0; al < ((Locus) sd.loci.get(loc)).number_alleles(); al++) {
                            ss += s_c_alleles[RN][sd.run_number][cohrt][loc][al] * s_c_alleles[RN][sd.run_number][cohrt][loc][al] / (4 * chsize * chsize);
                        }
                        h += 1 - ss;
                    }
                    s_calculated_heterozygosity[RN][sd.run_number][cohrt] = h / sd.number_loci;
                }
            }

            //We need that same thing --- calculated H ---for population at each year
            //new 30 sept 2006
            for (int ourage = 0; ourage < sd.number_generations + 1; ourage++) {
                float psize = s_p_population[RN][sd.run_number][ourage];
                if (psize > 0) {
                    float h = 0;
                    for (int loc = 0; loc < sd.number_loci; loc++) {
                        float ss = 0;
                        for (int al = 0; al < ((Locus) sd.loci.get(loc)).number_alleles(); al++) {
                            ss += s_p_alleles[RN][sd.run_number][ourage][loc][al] * s_p_alleles[RN][sd.run_number][ourage][loc][al] / (4 * psize * psize);
                        }
                        h += 1 - ss;
                    }
                    s_p_calculated_heterozygosity[RN][sd.run_number][ourage] = h / sd.number_loci;
                }
            }
        }


    }

    /**
     *showSummary() produces a String that gives all the stats
     * 19 Jan 08 make changes to deliver simple csv file, introducing
     * 2 extra variables into each line. One is cohort (true/false) and the
     * other is subregion (0,1,2...)
     */
    String showSummary(int subregion) {
        StringBuffer[] ba = new StringBuffer[sd.number_generations];
        for (int b = 0; b < sd.number_generations; b++) {
            ba[b] = new StringBuffer("true," + subregion + "," + b + ",");
        }
        for (int run = 0; run < sd.number_runs; run++) {
            for (int age = 0; age < sd.number_generations; age++) {
                for (int loc = 0; loc < sd.number_loci; loc++) {
                    for (int al = 0; al < sd.max_alleles; al++) {
                        if (c_alleles[run][age][loc][al] != 0) {
                            c_number_alleles[run][age]++;
                        }
                        if (p_alleles[run][age][loc][al] != 0) {
                            p_number_alleles[run][age]++;
                        }
                    }
                }
            }
        }
        //Summary by cohort
        double[] x = new double[sd.number_runs];
        double[] results;
        //population size
        for (int age = 0; age < sd.number_generations; age++) {
            for (int run = 0; run < sd.number_runs; run++) {
                x[run] = c_population[run][age];
            }
            results = DoStats(x);
            ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]) + ",");
        }
        //number alleles
        for (int age = 0; age < sd.number_generations; age++) {
            for (int run = 0; run < sd.number_runs; run++) {
                x[run] = c_number_alleles[run][age];
            }
            results = DoStats(x);
            ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]) + ",");
        }

        //New Improved Allele counts where we only average in nonzero cohorts
        //but still do average them in if they're small??


        for (int age = 0; age < sd.number_generations; age++) {
            int ourcount = 0;
            for (int run = 0; run < sd.number_runs; run++) {
                if (c_number_alleles[run][age] > 0) {
                    ourcount++;
                }
            }
            double[] ourx = new double[ourcount];
            int thiscount = 0;
            for (int run = 0; run < sd.number_runs; run++) {
                if (c_number_alleles[run][age] > 0) {
                    ourx[thiscount++] = c_number_alleles[run][age];
                }
            }
            results = DoStats(ourx);
           // ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]) + ",");
        }

        //heterozygosity old version averages in 0's for empty cohorts
        for (int age = 0; age < sd.number_generations; age++) {
            for (int run = 0; run < sd.number_runs; run++) {
                x[run] = 0;
                if (c_population[run][age] != 0) {
                    x[run] = ((double) c_heterozygous[run][age]) / (c_population[run][age] * sd.number_loci);
                }
            }
            results = DoStats(x);
            ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]) + ",");
        }
/*
        //new improved heterozygosity
        for (int age = 0; age < sd.number_generations; age++) {
            int ourcount = 0;
            for (int run = 0; run < sd.number_runs; run++) {
                if (c_population[run][age] != 0) {
                    ourcount++;
                }
            }
            double[] ourx = new double[ourcount];
            int thiscount = 0;
            for (int run = 0; run < sd.number_runs; run++) {
                x[run] = 0;
                if (c_population[run][age] != 0) {
                    ourx[thiscount++] = ((double) c_heterozygous[run][age]) / (c_population[run][age] * sd.number_loci);
                }
            }

            results = DoStats(ourx);
          //  ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]) + ",");
        }
        */
        //Now find stats on calcualted_heterozygosity and F
        ///NEW
        /*Within a single run we can find Ne for each year by
         *\[ H1/H0 = (1-\frac{1}{2N_e})\]
         *\[ \frac{1}{2N_e} = 1- H1/H0 = \frac{H_0-H_1}{H_0}
         *so \[N_e = (1/2)\frac{H_0}{H_1-H_0}\]
         */
        for (int age = 0; age < sd.number_generations; age++) {
            int ourcount = 0;

            for (int run = 0; run < sd.number_runs; run++) {
                if (c_population[run][age] != 0) {
                    ourcount++;
                }
            }
            double[] ourx = new double[ourcount];
            double[] ourxf = new double[ourcount];
            double[] ourne = new double[ourcount];
            int thiscount = 0;
            for (int run = 0; run < sd.number_runs; run++) {
                // x[run]=0;
                if (calculated_heterozygosity[run][age] != 0) {
                    ourx[thiscount++] = ((double) calculated_heterozygosity[run][age]);
                    if (age >= 1) {    //Do we want calculated or actual heterozygosity here
                        //WE WANT ACTUAL?
                        double h0 = ((double) c_heterozygous[run][age - 1]) / (c_population[run][age - 1] * sd.number_loci);
                        double h1 = ((double) c_heterozygous[run][age]) / (c_population[run][age] * sd.number_loci);
                        ourne[thiscount - 1] = 0.5 * (h0) / (h0 - h1);
                    }
                    //F = (Hcalc-Hactual)/Hcalc
                    //Hactual is total number of heterozygous loci divided by the total
                    //number of loci examined (population* number loci)
                    ourxf[thiscount - 1] = ((ourx[thiscount - 1] - ((double) c_heterozygous[run][age]) / (c_population[run][age] * sd.number_loci))) / ourx[thiscount - 1];
                }
            }
            results = DoStats(ourx);
            ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]));
            results = DoStats(ourxf);
            ba[age].append("," + nf.format(results[0]) + "," + nf.format(results[1]));
            if(params.ShowNe)
            {
                results = DoStats(ourne);
                ba[age].append("," + nf.format(results[0]) + "," + nf.format(results[1]));
            }
        }
        //ENDNEW
        //Assemble the String response
        StringBuffer labels;
        if (subregion == 0) {
            if(!params.ShowNe)
            {
                 //labels = new StringBuffer("cohort,subregion,age,mean(pop),sd(pop),mean(alleles),sd(alleles),mean(alleles in nonzero runs),sd(),mean(H observed)," +
                 //   "sd(H),mean(H.obs in nonzer runs), sd, H.exptd,sd(),calculated F, sd");
                  labels = new StringBuffer("cohort,subregion,age,mean(pop),sd(pop),mean(alleles),sd(alleles),mean(H observed)," +
                    "sd(H observed), mean(H expected),sd(H expected),mean(Fcalc), sd(Fcalc)");

            }
            else
            {
                //labels = new StringBuffer("cohort,subregion,age,mean(pop),sd(pop),mean(alleles),sd(alleles),mean(alleles in nonzero runs),sd(),mean(H observed)," +
                //    "sd(H),mean(H.obs in nonzer runs), sd, H.exptd,sd(),Fcalc, sd(Fcalc),Ne, sd(Ne)");

                labels = new StringBuffer("cohort,subregion,age,mean(pop),sd(pop),mean(alleles),sd(alleles),mean(H observed)," +
                    "sd(H), H.exptd,sd(), Fcalc, sd(Fcalc),Ne, sd(Ne)");
            }

        } else {
            labels = new StringBuffer();
        }
        for (int a = 0; a < sd.number_generations; a++) {
            labels.append("\n" + ba[a].toString());
        }

        //Annual summaries of entire population

        // labels.append("\n\nStats for population alive in each year\nage,meanpop,sdpop,meanalleles,sdalleles,meanhetero,sdhetero,mean H in nonzero runs,sd,"+
        //         "calculated H,sd,calculated F, sd,Ne, sd"
        //         );
        for (int b = 0; b < sd.number_generations; b++) {
            ba[b] = new StringBuffer("false," + subregion + "," + b + ",");
        }

        //population size
        for (int age = 0; age < sd.number_generations; age++) {
            for (int run = 0; run < sd.number_runs; run++) {
                x[run] = p_population[run][age];
            }
            results = DoStats(x);
            ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]) + ",");
        }
        //number alleles
        for (int age = 0; age < sd.number_generations; age++) {
            for (int run = 0; run < sd.number_runs; run++) {
                x[run] = p_number_alleles[run][age];
            }
            results = DoStats(x);
            ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]) + ",");
        }

        //heterozygosity
        for (int age = 0; age < sd.number_generations; age++) {
            for (int run = 0; run < sd.number_runs; run++) {
                x[run] = 0;
                if (p_population[run][age] != 0) {
                    x[run] = ((double) p_heterozygous[run][age]) / (p_population[run][age] * sd.number_loci);
                }
            }
            results = DoStats(x);
            ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]) );
        }

/*
        //new improved heterozygosity
        for (int age = 0; age < sd.number_generations; age++) {
            int ourcount = 0;
            for (int run = 0; run < sd.number_runs; run++) {
                if (p_population[run][age] != 0) {
                    ourcount++;
                }
            }
            double[] ourx = new double[ourcount];
            int thiscount = 0;
            for (int run = 0; run < sd.number_runs; run++) {
                x[run] = 0;
                if (p_population[run][age] != 0) {
                    ourx[thiscount++] = ((double) p_heterozygous[run][age]) / (p_population[run][age] * sd.number_loci);
                }
            }

            results = DoStats(ourx);
            ba[age].append("" + nf.format(results[0]) + "," + nf.format(results[1]));
        }
        */
        //NEW 30 SEPT 06 F and Ne for population in each year.
        for (int age = 0; age < sd.number_generations; age++) {
            int ourcount = 0;

            for (int run = 0; run < sd.number_runs; run++) {
                if (p_population[run][age] != 0) {
                    ourcount++;
                }
            }
            double[] ourx = new double[ourcount];
            double[] ourxf = new double[ourcount];
            double[] ourne = new double[ourcount];
            int thiscount = 0;
            for (int run = 0; run < sd.number_runs; run++) {
                // x[run]=0;
                if (p_calculated_heterozygosity[run][age] != 0) {
                    ourx[thiscount++] = ((double) p_calculated_heterozygosity[run][age]);
                    if (age >= 1) {    //Do we want calculated or actual heterozygosity here
                        //WE WANT ACTUAL?
                        double h0 = ((double) p_heterozygous[run][age - 1]) / (p_population[run][age - 1] * sd.number_loci);
                        double h1 = ((double) p_heterozygous[run][age]) / (p_population[run][age] * sd.number_loci);
                        ourne[thiscount - 1] = 0.5 * (h0) / (h0 - h1);
                    }
                    //F = (Hcalc-Hactual)/Hcalc
                    //Hactual is total number of heterozygous loci divided by the total
                    //number of loci examined (population* number loci)
                    ourxf[thiscount - 1] = ((ourx[thiscount - 1] - ((double) p_heterozygous[run][age]) / (p_population[run][age] * sd.number_loci))) / ourx[thiscount - 1];
                }
            }
            results = DoStats(ourx);
            ba[age].append("," + nf.format(results[0]) + "," + nf.format(results[1]));
            results = DoStats(ourxf);
            ba[age].append("," + nf.format(results[0]) + "," + nf.format(results[1]));
            if(params.ShowNe)
            {
            results = DoStats(ourne);
            ba[age].append("," + nf.format(results[0]) + "," + nf.format(results[1]));
            }
        }


        for (int a = 0; a < sd.number_generations; a++) {
            labels.append("\n" + ba[a].toString());
        }
        return labels.toString();
    }

    public String showRegionalSummary() {

        int number_summary_regions = sd.summaryregions.Regions.size();
        int RN; // number of summary region we're working on.
        //for each summary region
        StringBuffer ourAnswer = new StringBuffer();
        for (RN = 0; RN < number_summary_regions; RN++) {
            //  ourAnswer.append("\n\nResults for subregion "+RN+"\n");
            c_alleles = s_c_alleles[RN];
            c_population = s_c_population[RN];
            c_heterozygous = s_c_heterozygous[RN];
            c_number_alleles = s_c_number_alleles[RN];
            p_alleles = s_p_alleles[RN];
            p_population = s_p_population[RN];
            p_heterozygous = s_p_heterozygous[RN];
            p_number_alleles = s_p_number_alleles[RN];
            calculated_heterozygosity = s_calculated_heterozygosity[RN];
            p_calculated_heterozygosity = s_p_calculated_heterozygosity[RN];
            ourAnswer.append(showSummary(RN + 1));
        }
        //copy the numbers from the regional variables
        //to the global variables
        //and call showSummary()
        return ourAnswer.toString();
    }
/**
 *
 * @param x array to calculate mean and sd of;
 * @return an array with mean and standard deviation
 */
    double[] DoStats(double[] x) {
        double s = 0, ss = 0;
        double[] results = new double[2];
        //if we have no or smal samples
        if (x == null || x.length == 0) {
            results[0] = results[1] = 0.0;
        }
        if (x.length == 1) {
            results[0] = x[0];
            results[1] = 0.0;
        }
        //otherwise
        for (int i = 0; i < x.length; i++) {
            s += x[i];
            ss += (x[i] * x[i]);
        }
        results[0] = s / x.length;
        results[1] = Math.sqrt((ss - x.length * results[0] * results[0]) / (x.length - 1));
        return results;
    }

    /**
     *int locateClass(float x, float[] d)
     *is given a value $x$ and an array of values d[0],d[1],...,d[d.length-1]
     *these inputs are constants and not altered by this procedure
     *
     *The routine returns an integer i so that exactly one of 3 cases holds
     *
     *1) i==0 and x \le d[0]
     *2) 0<i<d.length and d[i-1]< x \le d[i]
     *3) i=d.length and x > d[d.length-1]
     *
     *
     *This should, of course, just be inline but I've
     *separated it out to include all the assertions and
     * for purposes of profiling
     */
    int locateClass(double x, float[] d) {
        int i = 0;
        while (i < d.length && x > d[i]) {
            i++;
        }

        if (DEBUG)//we can turn assertions off at runtime, but all this sort
        //of stuff I'll put in DEBUGs
        {
            assert (i == 0 && x <= d[0]) ||
                    (0 < i && i < d.length && d[i - 1] < x && x <= d[i]) ||
                    (i == d.length && x > d[d.length - 1]) :
                    "locateClass failed: x= " + x + " i = " + i;

        }

        return i;
    }

    int locateClass(double x, int[] d) {
        int i = 0;
        while (i < d.length && x > d[i]) {
            i++;
        }

        if (DEBUG)//we can turn assertions off at runtime, but all this sort
        //of stuff I'll put in DEBUGs
        {
            assert (i == 0 && x <= d[0]) ||
                    (0 < i && i < d.length && d[i - 1] < x && x <= d[i]) ||
                    (i == d.length && x > d[d.length - 1]) :
                    "locateClass failed: x= " + x + " i = " + i;

        }

        return i;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length <= 0) {
            System.out.println("This program needs input parameters. Provide the name\n" +
                               "of a SimData.dtd XML file as an argument on the command line\n");
        }
        Model t = new Model(args);
    }
}

class CommandLineParameters {

    String ProgramName = "NewGarden version 2.01 (10 Jan 2010)";
    String AuthorInfo = "info: Steve Pelikan pelikan@math.uc.edu";
    String RunDateTime;
    String XMLfilename;
    String DUMPfilename;
    String RESULTSfilename;
    boolean ShowProgress;
    PrintWriter dumpfilewriter;
    PrintWriter lifetalefilewriter;
    String LIFETABLEfilename;
    boolean ShowVersion;
    boolean ShowDump;
    boolean ShowXML;
    boolean ShowLifetable;
    boolean ShowNe;

    public CommandLineParameters() {
        Calendar rightNow = Calendar.getInstance();
        RunDateTime = rightNow.getTime().toString();
        ShowVersion = false;
        ShowDump = false;
        ShowXML = false;
        ShowNe=false;
        ShowLifetable = false;
        ShowProgress = false;
        XMLfilename = null;
        DUMPfilename = null;
        LIFETABLEfilename = null;
        RESULTSfilename = null;
    }
    //if s (stands for "short") is true, we only
    //put a little stuff in the String
    //otherwise, everything goes in.

    public String toString(boolean s) {
        StringBuilder sb = new StringBuilder();
        sb.append(ProgramName + "\n" + AuthorInfo + "\n");
        sb.append("Run on " + RunDateTime + "\n");
        if (!s) {
            sb.append("XMLfilename = " + XMLfilename + "\n");
            sb.append("Dump = " + ShowDump + " / DUMPfilename = " + DUMPfilename + "\n");
            sb.append("Lifetable = " + ShowLifetable + " / LIFETABLEfilename = " + LIFETABLEfilename + "\n");
            sb.append("ShowVersion = " + ShowVersion + "\n");
        }
        return sb.toString();
    }
}
