/*
 * Plant.java
 *
 * Created on February 28, 2005, 6:50 PM
/* Copyright  2005-2011 Stephan Pelikan and Steven Rogstad
/* contact:  pelikan@math.uc.edu
/*
/* This program is free software; you can redistribute it and/or modify
/* it under the terms of the GNU Library General Public License as published 
/* by  the Free Software Foundation; either version 3 of the License or
/* (at your option) any later version.
/*
/* This program is distributed in the hope that it will be useful, but
/* WITHOUT ANY WARRANTY; without even the implied warranty of
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/* GNU Library General Public License for more details.
/*
/* You should have received a copy of the GNU  General Public License
/* along with this program; see the file gpl-3.0.txt  If not, write to 
/* the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
/* Boston, MA  02111-1307 USA
 */

/**
 * A structure for containing all the information we need about a plant
 * @author sep
 */
public class Plant {
    /**
     * id_no is a class variable that lets us give each plant created a unique id*/
    static int id_no=0;
    /**my_id_no is the unique id for a given instance
     **/
    int my_id_no;
    /**parent1 and parent2 are the id numbers of the parents of an instance*/
    int parent1,parent2;
    /** Creates a new instance of Plant */
    public Plant() {
        my_id_no=(id_no++);
    }
    
    /**
     * whether plant is female or not
     */
    public boolean female;
    /**
     *wheether a plant was one of the founders of the population
     */
    public boolean founder;
    /**
     * an array that gives the alleles this plant has at the diferent loci
     */
    public  int [][]allele;
    /**
     * date of birth,death
     */
    public int dob,dod=-1;
    /**
     * location of the plant
     */
    public Location location;
    /**
     * create a plant with space of n loci
     * @param number_loci the number of loci we're tracking
     */
    public Plant(int number_loci){  my_id_no=(id_no++);allele=new int[number_loci][2];founder=false;}
    /**
     * create a plant
     * @param number_loci number of loci
     * @param db date of birth
     * @param l location
     */
    public Plant(int number_loci,int db,Location l) {
        my_id_no=(id_no++);
        allele=new int[number_loci][2];
        dob=db;
        location=l;
        founder=false;
    }
    /**
     *toXML() returns a String with data that can be used in the Simulation_Data .dtd
     *to describe the plant. The assignment of alleles isn't preserved but 
     * location, sex, and age are
     * @return a String in XML for this structure
     */
    public String toXML() {
        StringBuffer sb=new StringBuffer("");
        sb.append("<Plant age=\""+dob+"\" femaleP=\""+female+"\" X=\""+location.X+"\" Y=\""+location.Y+"\"/>\n");
        return sb.toString();
    }
    /**
     * a printable string describing this instance
     * @return String description of this instance of the class
     */
    @Override
    public String toString(){
        StringBuffer sb=new StringBuffer();
        sb.append("id_no="+my_id_no+"parent1 ="+parent1+" parent2 ="+parent2+"\ndob = "+dob+"\ndod = "+dod+"\n"+"female = "+female+"\n");
        sb.append("founder ="+founder+"\n");
        sb.append(location.toString()+"\nalleles\n");
        for(int i=0;i<allele.length;i++) {
            sb.append("locus "+i+":"+"("+allele[i][0]+","+allele[i][1]+")");
            sb.append("\n");
        }
        return sb.toString();
    }
    /**
     *toCSVString generates a single line of csv data about this plant
     *it is intended to be imported into a statistical package
     *The field names are printed in the order
     *ID,p1,p2,dob,female,locationX,locationY,loc_0_a_0,l_0_a_1,...locus_k_allele_m,...
     */
    public String toCSVString()
    {
        StringBuffer sb=new StringBuffer("");
         sb.append(""+0+",");
        sb.append(""+my_id_no+",");
        sb.append(""+parent1+",");
        sb.append(""+parent2+",");
        sb.append(""+dob+",");
        sb.append(""+dod+",");
        sb.append(""+female+",");
        sb.append(""+founder+",");
        sb.append(""+location.X+",");
        sb.append(""+location.Y+",");
        for(int loc=0;loc<allele.length;loc++)
        {
            for(int a=0;a<2;a++)
            {
                sb.append(""+allele[loc][a]+",");
            }
        }
        sb.append("\n");
        return sb.toString();
    }
    public String toCSVString(int run)
    {
        StringBuffer sb=new StringBuffer("");
        sb.append(""+run+",");
        sb.append(""+my_id_no+",");
        sb.append(""+parent1+",");
        sb.append(""+parent2+",");
        sb.append(""+dob+",");
        sb.append(""+dod+",");
        sb.append(""+female+",");
        sb.append(""+founder+",");
        sb.append(""+location.X+",");
        sb.append(""+location.Y+",");
        for(int loc=0;loc<allele.length;loc++)
        {
            for(int a=0;a<2;a++)
            {
                sb.append(""+allele[loc][a]+",");
            }
        }
        sb.append("\n");
        return sb.toString();
    }
    public String CSVHeader()
    {
         StringBuffer sb=new StringBuffer("");
         sb.append("run,ID,p1,p2,dob,dod,female,founder,X,Y,");
          for(int loc=0;loc<allele.length;loc++)
        {
            for(int a=0;a<2;a++)
            {
                sb.append("l"+loc+"a"+a+",");
            }
        }
        sb.append("\n");
        return sb.toString();
    }
}


