/*
 * SampledFunction.java
 *
/* Copyright  2005-2011 Stephan Pelikan and Steven Rogstad
/* contact:  pelikan@math.uc.edu
/*
/* This program is free software; you can redistribute it and/or modify
/* it under the terms of the GNU Library General Public License as published 
/* by  the Free Software Foundation; either version 3 of the License or
/* (at your option) any later version.
/*
/* This program is distributed in the hope that it will be useful, but
/* WITHOUT ANY WARRANTY; without even the implied warranty of
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/* GNU Library General Public License for more details.
/*
/* You should have received a copy of the GNU  General Public License
/* along with this program; see the file gpl-3.0.txt  If not, write to 
/* the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
/* Boston, MA  02111-1307 USA
 * Created on April 23, 2005, 10:06 PM
 */

import java.text.*;

/**
 * SampledFunction does PL interpolation
 * @author sep
 */
public class SampledFunction {

    public static final boolean DEBUG = false;
    public double[] x;
    public double[] y;

    /*
    //some functions to declare, test, enforce monotonicity
    //not currently used of fully implemented
    private boolean MonotoneIncreasing=false;
    private boolean MonotoneDecreasing=false;
    public void setMonotoneIncreasing(boolean v)
    {
    MonotoneIncreasing=v;
    assert this.monotoneIncreasing() : "SampledFunction is not monotone increasing in setMonotoneIncreasing";
    }
    public boolean getMonotoneIncreasing(){return (MonotoneIncreasing);}
    public void setMonotoneDecreasing(boolean v)
    {
    MonotoneDecreasing=v;
    assert this.monotoneDecreasing() : "SampledFunction is not monotone decreasing in setMonotoneDecreasing";
    }
    public boolean getMonotoneIDecreasing(){return (MonotoneDecreasing);}
     */
    /** Creates a new instance of SampledFunction */
    public SampledFunction() {
        if (DEBUG) {
            System.out.println("SampledFunction() called");

        }
        x = null;
        y = null;
    }

    public SampledFunction(double[] xx, double[] yy) {
        if (DEBUG) {
            //System.out.println("SampledFunction(double[],double[]) called");
            if (xx.length != yy.length) {
                System.out.println("SampledFunction ERROR: XX and YY have different lengths.");
            }

        }
        x = xx;
        y = yy;
        if (DEBUG) {
            if (!checkXMonotone()) {
                System.out.println("SampledFunction ERROR: X not monotone");
            }
        }
        //if WE ARE SUPPOSED TO BE MONOTONE, CHECK IF WE ARE
    }

    public boolean checkXMonotone() {
        boolean mono = true;
        if (x == null) {
            return true;
        }
        int i = 0;
        while (i < x.length - 1 && mono) {
            if (x[i] >= x[i + 1]) {
                mono = false;
            }
            i++;
        }
        return mono;
    }

    public boolean monotoneIncreasing() {
        if (y == null) {
            return true;
        }
        boolean mono = true;
        int i = 0;
        while (i < y.length - 1 && mono) {
            if (y[i] >= y[i + 1]) {
                mono = false;
            }
            i++;
        }
        return mono;
    }

    public boolean monotoneDecreasing() {
        if (y == null) {
            return true;
        }
        boolean mono = true;
        int i = 0;
        while (i < y.length - 1 && mono) {
            if (y[i] <= y[i + 1]) {
                mono = false;
            }
            i++;
        }
        return mono;
    }

    // public double [] getX(){return x;}
    //public double [] getY(){return y;}
    public double value(double t) {

        int i = 0;
        while (i <= x.length - 1 && x[i] <= t) {
            i++;
        }
        int j = i - 1;
        if (i <= 0) {
            return y[0];
        }
        if (i >= x.length) {
            return y[y.length - 1];
        } else {
            return y[j] + (y[i] - y[j]) * (t - x[j]) / (x[i] - x[j]);
        }
    }

    public double[] values(double[] t) {
        double[] ans = new double[t.length];
        for (int i = 0; i < t.length; i++) {
            ans[i] = value(t[i]);
        }
        return ans;
    }

    public double maximum_value() {
        double m = y[0];
        for (int i = 1; i < y.length; i++) {
            if (y[i] > m) {
                m = y[i];
            }
        }
        return m;
    }

    public double minimum_value() {
        double m = y[0];
        for (int i = 1; i < y.length; i++) {
            if (y[i] < m) {
                m = y[i];
            }
        }
        return m;
    }

    public void addPoint(double t, double z) {
        if (x == null || y == null) {
            x = new double[1];
            y = new double[1];
            x[0] = t;
            y[0] = z;
            return;
        }
        double[] newx = new double[x.length + 1];
        double[] newy = new double[y.length + 1];
        int i = 0;
        while (i < x.length && x[i] <= t)// changed < tp <= 8 July 2006
        {
            newx[i] = x[i];
            newy[i] = y[i];
            i++;
        }
        newx[i] = t;
        newy[i] = z;
        while (i < x.length) {
            newx[i + 1] = x[i];
            newy[i + 1] = y[i];
            i++;
        }
        x = newx;
        y = newy;
        //IF WE ARE SUPPOSED TO BE MONOTONE, CHECK IF WE ACTUALLY ARE
    }

    @Override
    public String toString() {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(2);
        StringBuffer sb = new StringBuffer("Number points = " + x.length + "\n");
        //sb.append("Monotonicity:"+MonotoneDecreasing+" "+MonotoneIncreasing);
        for (int i = 0; i < x.length; i++) {
            sb.append("(" + nf.format(x[i]) + "," + nf.format(y[i]) + ")\n");
        }
        return sb.toString();
    }

    public String toXML() {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(2);
        StringBuffer sb = new StringBuffer("<SampledFunction>" + "\n");
        for (int i = 0; i < x.length; i++) {
            sb.append("<functionpoint x=\"" + nf.format(x[i]) + "\" y=\"" + nf.format(y[i]) + "\"/>\n");
        }
        sb.append("</SampledFunction>");
        return sb.toString();
    }

    public String toXML(String name) {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(2);
        StringBuffer sb = new StringBuffer("<" + name + ">" + "\n");
        for (int i = 0; i < x.length; i++) {
            sb.append("<functionpoint x=\"" + nf.format(x[i]) + "\" y=\"" + nf.format(y[i]) + "\"/>\n");
        }
        sb.append("</" + name + ">\n");
        return sb.toString();
    }
}
