/*SimData. java
part of the program NewGarden
/* Copyright  2005-2011 Stephan Pelikan and Steven Rogstad
/* contact:  pelikan@math.uc.edu
/*
/* This program is free software; you can redistribute it and/or modify
/* it under the terms of the GNU Library General Public License as published 
/* by  the Free Software Foundation; either version 3 of the License or
/* (at your option) any later version.
/*
/* This program is distributed in the hope that it will be useful, but
/* WITHOUT ANY WARRANTY; without even the implied warranty of
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/* GNU Library General Public License for more details.
/*
/* You should have received a copy of the GNU  General Public License
/* along with this program; see the file gpl-3.0.txt  If not, write to 
/* the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
/* Boston, MA  02111-1307 USA
*/
import java.util.*;

public class SimData {

    boolean dioeceous;
    boolean random_mating;
    ArrayList loci;
    ArrayList plants;
    int max_alleles, number_loci;
    SampledFunction fruit_production;
    SampledFunction pollen_production;
    SampledFunction mortality;
    float selfing_rate;
    float foundersf;
    float fruit_distance, pollen_distance;
    CDF dispersal_distribution;
    Region region;
    SummaryRegions summaryregions;
    String off_dist_method;
    int number_generations, number_runs;
    int current_year, run_number;
    //new pollen stuff 7 april 06
    int[] pollen_distances;
    DiscreteProbabilityDistribution pollen_distance_probabilities;
    //end new pollen stuff

    public SimData() {
        loci = new ArrayList();
        plants = new ArrayList();
        fruit_production = new SampledFunction();
        pollen_production = new SampledFunction();
        mortality = new SampledFunction();
        dispersal_distribution = new CDF();
        current_year = 0;
        run_number = 0;
        //default values that should be over written by reading paramter file
        off_dist_method = "round";
        dioeceous = false;
        summaryregions = new SummaryRegions();
    }

    public void MakeFinal() {
        loci.trimToSize();
        plants.trimToSize();
        number_loci = loci.size();
        Iterator it = loci.iterator();
        max_alleles = 0;
        while (it.hasNext()) {
            Locus l = (Locus) it.next();
            // l.setF(this.foundersf);
            if (max_alleles < l.number_alleles()) {
                max_alleles = l.number_alleles();
            }
        }
    }

    @Override
    /**
     * Returns a string that is an XML version of the values
     * stored in this object.
     */
    public String toString() {
        StringBuilder sb = new StringBuilder("");
        sb.append("<LOCI  number_loci =\"" + loci.size() + "\">\n");
        Iterator it = loci.iterator();
        while (it.hasNext()) {
            Locus l = (Locus) it.next();
            sb.append(l.toXML());
        }
        sb.append("</LOCI>\n");
        sb.append("<Dioecious value =\"" + dioeceous + "\"/>\n");
        //  sb.append("\nReproduction_Rate\n"+fruit_production.toXML());
        //sb.append("\nReproduction_Rate\n"+fruit_production.toXML("Reproduction_Rate"));
        sb.append("\n" + fruit_production.toXML("Reproduction_Rate"));
        sb.append("\n<Offspring_Distribution method=\"" + off_dist_method + "\"/>\n");
        // sb.append("\nPollen_Rate\n"+pollen_production.toXML());
        sb.append("\n" + pollen_production.toXML("Pollen_Rate"));
        //  sb.append("\nMortality_Rate\n"+mortality.toXML());
        sb.append("\n" + mortality.toXML("Mortality_Rate"));
        sb.append("\n<Selfing_Rate value =\"" + selfing_rate + "\"/>\n");
        sb.append("\n<RandomMating value=\"" + random_mating + "\"/>\n");
        sb.append("<Fruit_Distance value =\"" + fruit_distance + "\"/>\n");
        //sb.append("\nDisperal_Distribution:\n"+ dispersal_distribution.toXML());
        sb.append("\n" + dispersal_distribution.toXML("Dispersal_Distribution"));

        //don't use this anymore
        // sb.append("<Pollen_Distance value =\""+pollen_distance+"\"/>\n");
        // sb.append("\nPollen_Distances:\n");
        sb.append("<Pollen_Distances>\n");
        double[] tt = new double[pollen_distances.length];
        double[] yy = new double[pollen_distances.length];
        for (int i = 0; i < tt.length; i++) {
            tt[i] = i;
            yy[i] = pollen_distances[i];
        }
        SampledFunction sf = new SampledFunction(tt, yy);
        sb.append(sf.toXML());
        sb.append("</Pollen_Distances>\n");

        //  sb.append("\nPollen_Distance_Probabilities:\n");
        sb.append("<Pollen_Distance_Probabilities>\n");

        sb.append(pollen_distance_probabilities.toXML());
        sb.append("</Pollen_Distance_Probabilities>\n");
        //Region
        sb.append(region.toXML());
        sb.append(this.summaryregions.toXML());
        sb.append("<Number_Generations value =\"" + number_generations + "\"/>\n");
        sb.append("<Number_Runs value =\"" + number_runs + "\"/>\n");
        sb.append("<Initial_Population>\n");
        it = plants.iterator();
        while (it.hasNext()) {
            Plant p = (Plant) it.next();
            sb.append(p.toXML());
        }
        sb.append("</Initial_Population>\n");
        sb.append("<FoundersF value =\"" + foundersf + "\"/>\n");
        return sb.toString();
    }
}
